@@ -2,7 +2,7 @@
 #'
 #'Compute the root mean square error skill score (RMSSS) between an array of 
 #'forecast 'exp' and an array of observation 'obs'. The two arrays should 
-#'have the same dimensions except along memb_dim, where the length can be 
+#'have the same dimensions except along dat_dim, where the length can be 
 #'different, with the number of experiments/models (nexp) and the number of 
 #'observational datasets (nobs).\cr
 #'RMSSS computes the root mean square error skill score of each jexp in 1:nexp 
@@ -13,13 +13,17 @@
 #'The p-value is optionally provided by an one-sided Fisher test.\cr
 #'
 #'@param exp A named numeric array of experimental data which contains at least
-#'  two dimensions for memb_dim and time_dim.
+#'  two dimensions for dat_dim and time_dim. It can also be a vector with the 
+#'  same length as 'obs', then the vector will automatically be 'time_dim' and 
+#'  'dat_dim' will be 1.
 #'@param obs A named numeric array of observational data which contains at least
-#'  two dimensions for memb_dim and time_dim. The dimensions should be the same 
-#'  as paramter 'exp' except the length of 'memb_dim' dimension. The order of 
-#'  dimension can be different.
-#'@param memb_dim A character string indicating the name of member (nobs/nexp) 
-#'  dimension. The default value is 'member'.
+#'  two dimensions for dat_dim and time_dim. The dimensions should be the same 
+#'  as paramter 'exp' except the length of 'dat_dim' dimension. The order of 
+#'  dimension can be different. It can also be a vector with the same length as
+#'  'exp', then the vector will automatically be 'time_dim' and 'dat_dim' will
+#'  be 1.
+#'@param dat_dim A character string indicating the name of dataset (nobs/nexp) 
+#'  dimension. The default value is 'dataset'.
 #'@param time_dim A character string indicating the name of dimension along  
 #'  which the RMSSS are computed. The default value is 'sdate'.
 #'@param pval A logical value indicating whether to compute or not the p-value 
@@ -31,8 +35,8 @@
 #'@return
 #'A list containing the numeric arrays with dimension:\cr 
 #'  c(nexp, nobs, all other dimensions of exp except time_dim).\cr
-#'nexp is the number of experiment (i.e., memb_dim in exp), and nobs is the 
-#'number of observation (i.e., memb_dim in obs).\cr
+#'nexp is the number of experiment (i.e., dat_dim in exp), and nobs is the 
+#'number of observation (i.e., dat_dim in obs).\cr
 #'\item{$rmsss}{
 #'  The root mean square error skill score. 
 #'}
@@ -42,16 +46,16 @@
 #'
 #'@examples
 #' set.seed(1)
-#' exp <- array(rnorm(15), dim = c(dat = 1, time = 3, member = 5))
+#' exp <- array(rnorm(30), dim = c(dataset = 2, time = 3, memb = 5))
 #' set.seed(2)
-#' obs <- array(rnorm(6), dim = c(time = 3, member = 2, dat = 1))
+#' obs <- array(rnorm(15), dim = c(time = 3, memb = 5, dataset = 1))
 #' res <- RMSSS(exp, obs, time_dim = 'time')
 #'
 #'@rdname RMSSS
 #'@import multiApply
 #'@importFrom stats pf
 #'@export
-RMSSS <- function(exp, obs, time_dim = 'sdate', memb_dim = 'member', 
+RMSSS <- function(exp, obs, time_dim = 'sdate', dat_dim = 'dataset', 
                   pval = TRUE, ncores = NULL) {
   
   # Check inputs 
@@ -62,9 +66,19 @@ RMSSS <- function(exp, obs, time_dim = 'sdate', memb_dim = 'member',
   if (!is.numeric(exp) | !is.numeric(obs)) {
     stop("Parameter 'exp' and 'obs' must be a numeric array.")
   }
-  if (is.null(dim(exp)) | is.null(dim(obs))) {
-    stop(paste0("Parameter 'exp' and 'obs' must be at least two dimensions ",
-                "containing time_dim and memb_dim."))
+  if (is.null(dim(exp)) & is.null(dim(obs))) {  #is vector
+    if (length(exp) == length(obs)) {
+      exp <- array(exp, dim = c(length(exp), 1))
+      names(dim(exp)) <- c(time_dim, dat_dim)
+      obs <- array(obs, dim = c(length(obs), 1))
+      names(dim(obs)) <- c(time_dim, dat_dim)
+    } else {
+    stop(paste0("Parameter 'exp' and 'obs' must be array with as least two ",
+                "dimensions time_dim and dat_dim, or vector of same length."))
+    }
+  } else if (is.null(dim(exp)) | is.null(dim(obs))) {
+    stop(paste0("Parameter 'exp' and 'obs' must be array with as least two ",
+                "dimensions time_dim and dat_dim, or vector of same length."))
   }
   if(any(is.null(names(dim(exp))))| any(nchar(names(dim(exp))) == 0) |
      any(is.null(names(dim(obs))))| any(nchar(names(dim(obs))) == 0)) {
@@ -81,12 +95,12 @@ RMSSS <- function(exp, obs, time_dim = 'sdate', memb_dim = 'member',
   if (!time_dim %in% names(dim(exp)) | !time_dim %in% names(dim(obs))) {
     stop("Parameter 'time_dim' is not found in 'exp' or 'obs' dimension.")
   }
-  ## memb_dim
-  if (!is.character(memb_dim) | length(memb_dim) > 1) {
-    stop("Parameter 'memb_dim' must be a character string.")
+  ## dat_dim
+  if (!is.character(dat_dim) | length(dat_dim) > 1) {
+    stop("Parameter 'dat_dim' must be a character string.")
   }
-  if (!memb_dim %in% names(dim(exp)) | !memb_dim %in% names(dim(obs))) {
-    stop("Parameter 'memb_dim' is not found in 'exp' or 'obs' dimension.")
+  if (!dat_dim %in% names(dim(exp)) | !dat_dim %in% names(dim(obs))) {
+    stop("Parameter 'dat_dim' is not found in 'exp' or 'obs' dimension.")
   }
   ## pval
   if (!is.logical(pval) | length(pval) > 1) {
@@ -102,11 +116,11 @@ RMSSS <- function(exp, obs, time_dim = 'sdate', memb_dim = 'member',
   ## exp and obs (2)
   name_exp <- sort(names(dim(exp)))
   name_obs <- sort(names(dim(obs)))
-  name_exp <- name_exp[-which(name_exp == memb_dim)]
-  name_obs <- name_obs[-which(name_obs == memb_dim)]
+  name_exp <- name_exp[-which(name_exp == dat_dim)]
+  name_obs <- name_obs[-which(name_obs == dat_dim)]
   if(!all(dim(exp)[name_exp] == dim(obs)[name_obs])) {
     stop(paste0("Parameter 'exp' and 'obs' must have same length of ",
-                "all dimension expect 'memb_dim'."))
+                "all dimension expect 'dat_dim'."))
   }
   if (dim(exp)[time_dim] <= 2) {
     stop("The length of time_dim must be more than 2 to compute RMSSS.")
@@ -125,47 +139,47 @@ RMSSS <- function(exp, obs, time_dim = 'sdate', memb_dim = 'member',
   # Calculate RMSSS
   
   res <- Apply(list(exp, obs), 
-               target_dims = list(c(time_dim, memb_dim), 
-                                  c(time_dim, memb_dim)),
+               target_dims = list(c(time_dim, dat_dim), 
+                                  c(time_dim, dat_dim)),
                fun = .RMSSS, 
-               time_dim = time_dim, memb_dim = memb_dim,
+               time_dim = time_dim, dat_dim = dat_dim,
                pval = pval, #conf = conf, conf.lev = conf.lev,
                ncores = ncores)
   
   return(res)
 }
 
-.RMSSS <- function(exp, obs, time_dim = 'sdate', memb_dim = 'member', pval = TRUE) {
-  # exp: [sdate, member_exp]
-  # obs: [sdate, member_obs]
-  n_exp <- as.numeric(dim(exp)[2])
-  n_obs <- as.numeric(dim(obs)[2])
-  n_sdate <- as.numeric(dim(exp)[1])
+.RMSSS <- function(exp, obs, time_dim = 'sdate', dat_dim = 'dataset', pval = TRUE) {
+  # exp: [sdate, dat_exp]
+  # obs: [sdate, dat_obs]
+  nexp <- as.numeric(dim(exp)[2])
+  nobs <- as.numeric(dim(obs)[2])
+  nsdate <- as.numeric(dim(exp)[1])
   
-  p_val <- array(dim = c(nexp = n_exp, nobs = n_obs))
-  dif1 <- array(dim = c(n_sdate, n_exp, n_obs))
+  p_val <- array(dim = c(nexp = nexp, nobs = nobs))
+  dif1 <- array(dim = c(nsdate, nexp, nobs))
   names(dim(dif1)) <- c(time_dim, 'nexp', 'nobs')
 
 #  if (conf) {
 #    conflow <- (1 - conf.lev) / 2
 #    confhigh <- 1 - conflow
-#    conf_low <- array(dim = c(nexp = n_exp, nobs = n_obs))
-#    conf_high <- array(dim = c(nexp = n_exp, nobs = n_obs))
+#    conf_low <- array(dim = c(nexp = nexp, nobs = nobs))
+#    conf_high <- array(dim = c(nexp = nexp, nobs = nobs))
 #  }
 
   # dif1
-  for (i in 1:n_obs) {
-    dif1[, , i] <- sapply(1:n_exp, function(x) {exp[, x] - obs[, i]})
+  for (i in 1:nobs) {
+    dif1[, , i] <- sapply(1:nexp, function(x) {exp[, x] - obs[, i]})
   }
 
   # rms1 and eno1
-  rms1 <- apply(dif1^2, c(2, 3), mean, na.rm = TRUE)^0.5  #array(dim = c(n_exp, n_obs))
+  rms1 <- apply(dif1^2, c(2, 3), mean, na.rm = TRUE)^0.5  #array(dim = c(nexp, nobs))
   # rms2 and eno2
-  rms2 <- array(colMeans(obs^2, na.rm = TRUE)^0.5, dim = c(n_obs = n_obs))
+  rms2 <- array(colMeans(obs^2, na.rm = TRUE)^0.5, dim = c(nobs = nobs))
   rms2[which(abs(rms2) <= (max(abs(rms2), na.rm = TRUE) / 1000))] <- max(abs(
     rms2), na.rm = TRUE) / 1000
   #rms2 above: [nobs]
-  rms2 <- array(rms2, dim = c(nobs = n_obs, nexp = n_exp))
+  rms2 <- array(rms2, dim = c(nobs = nobs, nexp = nexp))
   #rms2 above: [nobs, nexp]
   rms2 <- Reorder(rms2, c(2, 1))
   #rms2 above: [nexp, nobs]
@@ -177,7 +191,7 @@ RMSSS <- function(exp, obs, time_dim = 'sdate', memb_dim = 'member',
   if (pval) {
     eno1 <- Eno(dif1, time_dim)  
     eno2 <- Eno(obs, time_dim)  
-    eno2 <- array(eno2, dim = c(nobs = n_obs, nexp = n_exp))
+    eno2 <- array(eno2, dim = c(nobs = nobs, nexp = nexp))
     eno2 <- Reorder(eno2, c(2, 1))
   }
 
